{-# OPTIONS -fglasgow-exts -cpp #-}

{-
    Embedded interpreters.

    As Beren looked into her eyes
    Within the shadows of her hair,
    The trembling starlight of the skies
    He saw there mirrored shimmering...
-}

module Embed (
    module Embed.Perl5,
) where
import Embed.Perl5
-- import Embed.Haskell
-- import Embed.Parrot
-- import Embed.Ponie
